@@ -120,13 +120,13 @@ diff_computed_accumulator (GSignalInvocationHint *ihint,
         thisvalue = g_value_get_boolean (handler_return);
         g_value_set_boolean (return_accu, thisvalue);
 
-        return thisvalue; /* stop signal if 'thisvalue' is FALSE */
+        return !thisvalue; /* stop signal if 'thisvalue' is FALSE */
 }
 
 static gboolean
 m_diff_computed (GdaDataComparator *comparator, GdaDiff *diff)
 {
-        return TRUE; /* default is to allow differences computing to proceed */
+        return FALSE; /* default is to allow differences computing to proceed (understand it as: FALSE => don't stop) */
 }
 
 static void
@@ -139,12 +139,12 @@ gda_data_comparator_class_init (GdaDataComparatorClass *class)
 	/* signals */
 
 	gda_data_comparator_signals [DIFF_COMPUTED] =
-		g_signal_new ("diff_computed",
+		g_signal_new ("diff-computed",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GdaDataComparatorClass, diff_computed),
                               diff_computed_accumulator, NULL,
-                              gda_marshal_BOOLEAN__POINTER, G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
+                              _gda_marshal_BOOLEAN__POINTER, G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
 	class->diff_computed = m_diff_computed;
 
@@ -336,7 +336,7 @@ gda_diff_free (GdaDiff *diff)
  * will speed things up as less data will be processed.
  */
 void
-gda_data_comparator_set_key_columns (GdaDataComparator *comp, gint *col_numbers, gint nb_cols)
+gda_data_comparator_set_key_columns (GdaDataComparator *comp, const gint *col_numbers, gint nb_cols)
 {
 	g_return_if_fail (GDA_IS_DATA_COMPARATOR (comp));
 	g_return_if_fail (comp->priv);
@@ -355,6 +355,7 @@ gda_data_comparator_set_key_columns (GdaDataComparator *comp, gint *col_numbers,
  * It is assumed that both data model have the same number of columns and of "compatible" types.
  *
  * Returns: 
+ *          -2 if an error occurred
  *          -1 if not found, 
  *          >=0 if found (if changes need to be made, then @out_has_changed is set to TRUE).
  */
@@ -374,10 +375,16 @@ find_row_in_model (GdaDataComparator *comp, gint row, gboolean *out_has_changed,
 			comp->priv->key_columns [i] = i;
 	}
 
-	for (i = 0; i < comp->priv->nb_key_columns; i++) 
-		values = g_slist_append (values, 
-					 (gpointer) gda_data_model_get_value_at (comp->priv->new_model, 
-										 comp->priv->key_columns[i], row));
+	for (i = 0; i < comp->priv->nb_key_columns; i++) {
+		const GValue *cvalue;
+		cvalue = gda_data_model_get_value_at (comp->priv->new_model, comp->priv->key_columns[i], row, error);
+		if (!cvalue) {
+			if (values)
+				g_slist_free (values);
+			return -2;
+		}
+		values = g_slist_append (values, (gpointer) cvalue);
+	}
 
 	erow = gda_data_model_get_row_from_values (comp->priv->old_model, values, comp->priv->key_columns);
 	g_slist_free (values);
@@ -386,9 +393,13 @@ find_row_in_model (GdaDataComparator *comp, gint row, gboolean *out_has_changed,
 		gboolean changed = FALSE;
 		for (i = 0; i < ncols; i++) {
 			const GValue *v1, *v2;
-			v1 = gda_data_model_get_value_at (comp->priv->old_model, i, erow);
-			v2 = gda_data_model_get_value_at (comp->priv->new_model, i, row);
-			if (gda_value_compare_ext (v1, v2)) {
+			v1 = gda_data_model_get_value_at (comp->priv->old_model, i, erow, error);
+			if (!v1)
+				return -2;
+			v2 = gda_data_model_get_value_at (comp->priv->new_model, i, row, error);
+			if (!v2) 
+				return -2;
+			if (gda_value_compare (v1, v2)) {
 				changed = TRUE;
 				break;
 			}
@@ -410,7 +421,7 @@ find_row_in_model (GdaDataComparator *comp, gint row, gboolean *out_has_changed,
  * If one connects to this signal and returns FALSE in the signal handler, then computing differences will be
  * stopped and an error will be returned.
  *
- * Returns: TRUE if all the differences have been sucessfully computed
+ * Returns: TRUE if all the differences have been sucessfully computed, and FALSE if an error occurred
  */
 gboolean
 gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
@@ -427,18 +438,18 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 	/* check setup */
 	if (!comp->priv->old_model) {
 		g_set_error (error, GDA_DATA_COMPARATOR_ERROR, GDA_DATA_COMPARATOR_MISSING_DATA_MODEL_ERROR,
-			     _("Missing original data model"));
+			      "%s", _("Missing original data model"));
 		return FALSE;
 	}
 	if (!comp->priv->new_model) {
 		g_set_error (error, GDA_DATA_COMPARATOR_ERROR, GDA_DATA_COMPARATOR_MISSING_DATA_MODEL_ERROR,
-			     _("Missing new data model"));
+			      "%s", _("Missing new data model"));
 		return FALSE;
 	}
 	if (! (gda_data_model_get_access_flags (comp->priv->old_model) & GDA_DATA_MODEL_ACCESS_RANDOM) ||
 	    ! (gda_data_model_get_access_flags (comp->priv->new_model) & GDA_DATA_MODEL_ACCESS_RANDOM)) {
 		g_set_error (error, GDA_DATA_COMPARATOR_ERROR, GDA_DATA_COMPARATOR_MODEL_ACCESS_ERROR,
-			     _("Data models must support random access model"));
+			      "%s", _("Data models must support random access model"));
 		return FALSE;
 	}
 
@@ -447,7 +458,7 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 	nncols = gda_data_model_get_n_columns (comp->priv->new_model);
 	if (oncols != nncols) {
 		g_set_error (error, GDA_DATA_COMPARATOR_ERROR, GDA_DATA_COMPARATOR_MISSING_DATA_MODEL_ERROR,
-			     _("Data models to compare don't have the same number of columns"));
+			      "%s", _("Data models to compare don't have the same number of columns"));
 		return FALSE;
 	}
 
@@ -467,7 +478,9 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 
 	/* actual differences computations : rows to insert / update */
 	onrows = gda_data_model_get_n_rows (comp->priv->old_model);
+	g_assert (onrows >= 0);
 	nnrows = gda_data_model_get_n_rows (comp->priv->new_model);
+	g_assert (nnrows >= 0);
 	rows_to_del = g_new (gboolean, onrows);
 	memset (rows_to_del, TRUE, sizeof (gboolean) * onrows);
 	for (i = 0; i < nnrows; i++) {
@@ -476,13 +489,13 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 		GdaDiff *diff = NULL;
 		gboolean stop;
 		
-		erow = find_row_in_model (comp, i,  &has_changed, error);
+		erow = find_row_in_model (comp, i, &has_changed, error);
 		
 #ifdef DEBUG_STORE_MODIFY
-		g_print ("FIND row %d(/%d) returned row %d (%s)\n", i, new_n_rows - 1, erow, 
+		g_print ("FIND row %d returned row %d (%s)\n", i, erow, 
 			 has_changed ? "CHANGED" : "unchanged");
 #endif
-		if (erow < 0) {
+		if (erow == -1) {
 			gint j;
 			diff = g_new0 (GdaDiff, 1);
 			diff->type = GDA_DIFF_ADD_ROW;
@@ -490,10 +503,23 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 			diff->new_row = i;
 			diff->values = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free,
 							      (GDestroyNotify) gda_value_free);
-			for (j = 0; j < oncols; j++) 
+			for (j = 0; j < oncols; j++) {
+				const GValue *cvalue;
+				cvalue = gda_data_model_get_value_at (comp->priv->new_model, j, i, error);
+				if (!cvalue) {
+					/* an error occurred */
+					g_free (rows_to_del);
+					gda_diff_free (diff);
+					return FALSE;
+				}
 				g_hash_table_insert (diff->values, g_strdup_printf ("+%d", j),
-						     gda_value_copy (gda_data_model_get_value_at (comp->priv->new_model,
-												  j, i)));
+						     gda_value_copy (cvalue));
+			}
+		}
+		else if (erow < -1) {
+			/* an error occurred */
+			g_free (rows_to_del);
+			return FALSE;
 		}
 		else if (has_changed) {
 			gint j;
@@ -504,12 +530,25 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 			diff->values = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free,
 							      (GDestroyNotify) gda_value_free);
 			for (j = 0; j < oncols; j++) {
+				const GValue *cvalue;
+				cvalue = gda_data_model_get_value_at (comp->priv->new_model, j, i, error);
+				if (!cvalue) {
+					/* an error occurred */
+					g_free (rows_to_del);
+					gda_diff_free (diff);
+					return FALSE;
+				}
 				g_hash_table_insert (diff->values, g_strdup_printf ("+%d", j),
-						     gda_value_copy (gda_data_model_get_value_at (comp->priv->new_model,
-												  j, i)));
+						     gda_value_copy (cvalue));
+				cvalue = gda_data_model_get_value_at (comp->priv->old_model, j, i, error);
+				if (!cvalue) {
+					/* an error occurred */
+					g_free (rows_to_del);
+					gda_diff_free (diff);
+					return FALSE;
+				}
 				g_hash_table_insert (diff->values, g_strdup_printf ("-%d", j),
-						     gda_value_copy (gda_data_model_get_value_at (comp->priv->old_model,
-												  j, i)));
+						     gda_value_copy (cvalue));
 			}
 		}
 		rows_to_del [i] = FALSE;
@@ -519,7 +558,7 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 			if (stop) {
 				g_set_error (error, GDA_DATA_COMPARATOR_ERROR,
 					     GDA_DATA_COMPARATOR_USER_CANCELLED_ERROR,
-					     _("Differences computation cancelled on signal handling"));
+					      "%s", _("Differences computation cancelled on signal handling"));
 				g_free (rows_to_del);
 				return FALSE;
 			}
@@ -538,16 +577,24 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 			diff->new_row = -1;
 			diff->values = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free,
 							      (GDestroyNotify) gda_value_free);
-			for (j = 0; j < oncols; j++) 
+			for (j = 0; j < oncols; j++)  {
+				const GValue *cvalue;
+				cvalue = gda_data_model_get_value_at (comp->priv->old_model, j, i, error);
+				if (!cvalue) {
+					/* an error occurred */
+					g_free (rows_to_del);
+					gda_diff_free (diff);
+					return FALSE;
+				}
 				g_hash_table_insert (diff->values, g_strdup_printf ("-%d", j),
-						     gda_value_copy (gda_data_model_get_value_at (comp->priv->old_model,
-												  j, i)));
+						     gda_value_copy (cvalue));
+			}
 			g_array_append_val (comp->priv->diffs, diff);
 			g_signal_emit (comp, gda_data_comparator_signals [DIFF_COMPUTED], 0, diff, &stop);
 			if (stop) {
 				g_set_error (error, GDA_DATA_COMPARATOR_ERROR,
 					     GDA_DATA_COMPARATOR_USER_CANCELLED_ERROR,
-					     _("Differences computation cancelled on signal handling"));
+					      "%s", _("Differences computation cancelled on signal handling"));
 				g_free (rows_to_del);
 				return FALSE;
 			}
