@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 1998 - 2008 The GNOME Foundation.
+ * Copyright (C) 1998 - 2009 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -27,10 +27,6 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <string.h>
-#include <glib/gfileutils.h>
-#include <glib/gmessages.h>
-#include <glib/gstrfuncs.h>
-#include <glib/gprintf.h>
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-util.h>
@@ -44,19 +40,40 @@
 #include <libgda/binreloc/gda-binreloc.h>
 
 extern gchar *gda_lang_locale;
+extern GdaAttributesManager *gda_holder_attributes_manager;
 
 /**
  * gda_g_type_to_string
  * @type: Type to convert from.
  *
- * Returns: the string representing the given #GType.
+ * Converts a GType to its string representation (use gda_g_type_from_string() for the
+ * operation in the other direction).
+ *
+ * This function wraps g_type_name() but for common types it provides an easier to
+ * understand and remember name. For Example the G_TYPE_STRING is converted to "string"
+ * whereas g_type_name() converts it to "gchararray".
+ *
+ * Returns: the GDA's string representing the given #GType or the name
+ * returned by #g_type_name.
  */
 const gchar *
 gda_g_type_to_string (GType type)
 {
 	if (type == GDA_TYPE_NULL)
 		return "null";
-	else
+	else if (type == G_TYPE_INT)
+		return "int";
+	else if (type == G_TYPE_STRING)
+		return "string";
+	else if (type == G_TYPE_DATE)
+		return "date";			
+	else if (type == GDA_TYPE_TIME)
+		return "time";
+	else if (type == GDA_TYPE_TIMESTAMP)
+		return "timestamp";
+	else if (type == G_TYPE_BOOLEAN)
+		return "boolean";
+	else 
 		return g_type_name (type);
 }
 
@@ -64,6 +81,20 @@ gda_g_type_to_string (GType type)
  * gda_g_type_from_string
  * @str: the name of a #GType, as returned by gda_g_type_to_string().
  *
+ * Converts a named type to ts GType type (also see the gda_g_type_to_string() function).
+ *
+ * This function is a wrapper around the g_type_from_name() function, but also recognizes
+ * some type synonyms such as:
+ * <itemizedlist>
+ *   <listitem><para>"int" for G_TYPE_INT</para></listitem>
+ *   <listitem><para>"string" for G_TYPE_STRING</para></listitem>
+ *   <listitem><para>"date" for G_TYPE_DATE</para></listitem>
+ *   <listitem><para>"time" for GDA_TYPE_TIME</para></listitem>
+ *   <listitem><para>"timestamp" for GDA_TYPE_TIMESTAMP</para></listitem>
+ *   <listitem><para>"boolean" for G_TYPE_BOOLEAN</para></listitem>
+ *   <listitem><para>"null" for GDA_TYPE_NULL</para></listitem>
+ * </itemizedlist>
+ *
  * Returns: the #GType represented by the given @str.
  */
 GType
@@ -102,6 +133,8 @@ gda_g_type_from_string (const gchar *str)
  *
  * Escapes @string to make it understandable by a DBMS. The escape method is very common and replaces any
  * occurence of "'" with "''" and "\" with "\\"
+ *
+ * Returns: a new string
  */
 gchar *
 gda_default_escape_string (const gchar *string)
@@ -356,12 +389,12 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 				}
 				if (!use_col_ids) {
 					if (str && *str) 
-						field = xmlNewChild (row, NULL,  (xmlChar*)"gda_value", (xmlChar*)str);
+						field = xmlNewTextChild (row, NULL,  (xmlChar*)"gda_value", (xmlChar*)str);
 					else
 						field = xmlNewChild (row, NULL,  (xmlChar*)"gda_value", NULL);
 				}
 				else {
-					field = xmlNewChild (row, NULL,  (xmlChar*)"gda_array_value", (xmlChar*)str);
+					field = xmlNewTextChild (row, NULL,  (xmlChar*)"gda_array_value", (xmlChar*)str);
 					xmlSetProp(field, (xmlChar*)"colid",  (xmlChar*)col_ids [c]);
 				}
 
@@ -373,6 +406,9 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 		}
 	}
 
+	if(!cols)
+		g_free(rcols);
+
 	if (use_col_ids) {
 		gint c;
 		for (c = 0; c < rnb_cols; c++) 
@@ -388,6 +424,61 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 	return retval;
 }
 
+
+/**
+ * gda_utility_data_model_find_column_description
+ * @model: a #GdaDataSelect data model
+ * @field_name: field name
+ *
+ * Finds the description of a field into Metadata from a #GdaDataModel.
+ *
+ * Returns: The field's description, or NULL if description is not set
+ */
+const gchar *
+gda_utility_data_model_find_column_description (GdaDataSelect *model, const gchar *field_name)
+{
+	g_return_val_if_fail (GDA_IS_DATA_SELECT (model), NULL);
+	g_return_val_if_fail (field_name, NULL);
+
+	GdaConnection *connection = gda_data_select_get_connection ((GdaDataSelect *) model);
+
+	GdaStatement *statement;
+	g_object_get (G_OBJECT (model), "select-stmt", &statement, NULL);
+	if (!statement)
+		return NULL;
+
+	GdaSqlStatement *sql_statement;
+	g_object_get (G_OBJECT (statement), "structure", &sql_statement, NULL);
+	g_object_unref (statement);
+
+	if (!gda_sql_statement_check_validity (sql_statement, connection, NULL)) {
+		gda_sql_statement_free (sql_statement);
+		return NULL;
+	}
+
+	GSList *fields;
+	for (fields = ((GdaSqlStatementSelect *) sql_statement->contents)->expr_list;
+	     fields;
+	     fields = fields->next) {
+
+		GdaSqlSelectField *select_field = fields->data;
+		if (select_field->validity_meta_table_column) {
+			GdaMetaTableColumn *meta_table_column = select_field->validity_meta_table_column;
+
+			if (! strcmp (meta_table_column->column_name, field_name)) {
+				const GValue *gvalue = gda_meta_table_column_get_attribute
+					(meta_table_column, GDA_ATTRIBUTE_DESCRIPTION);
+
+				gda_sql_statement_free (sql_statement);
+				return gvalue ? g_value_get_string (gvalue) : NULL;
+			}
+		}
+	}
+
+	gda_sql_statement_free (sql_statement);
+	return NULL;
+}
+
 /**
  * gda_utility_holder_load_attributes
  * @holder:
@@ -430,9 +521,6 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 	}
 	else
 		gda_holder_set_not_null (holder, FALSE);
-	str = xmlGetProp (node, BAD_CAST "plugin");
-	if (str) 
-		g_object_set_data_full (G_OBJECT (holder), "__gda_entry_plugin", str, xmlFree);
 	
 	str = xmlGetProp (node, BAD_CAST "source");
 	if (str) 
@@ -495,6 +583,22 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 				continue;
 			}
 
+			if (!strcmp ((gchar*)vnode->name, "attribute")) {
+				xmlChar *att_name;
+				att_name = xmlGetProp (vnode, (xmlChar*) "name");
+				if (att_name) {
+					GValue *value;
+					g_value_set_string ((value = gda_value_new (G_TYPE_STRING)), 
+							    (gchar*) xmlNodeGetContent (vnode));
+					gda_attributes_manager_set_full (gda_holder_attributes_manager,
+									 (gpointer) holder,
+									 (gchar*) att_name, value,
+									 (GDestroyNotify) xmlFree);
+					gda_value_free (value);
+				}
+				vnode = vnode->next;
+				continue;
+			}
 			if (strcmp ((gchar*)vnode->name, "gda_value")) {
 				vnode = vnode->next;
 				continue;
@@ -575,7 +679,7 @@ gda_text_to_alphanum (const gchar *text)
 
 /**
  * gda_alphanum_to_text
- * @text:
+ * @text: a string
  *
  * Does the opposite of gda_text_to_alphanum(), in the same string 
  *
@@ -634,19 +738,19 @@ dml_statements_check_select_structure (GdaConnection *cnc, GdaSqlStatement *sel_
 	stsel = (GdaSqlStatementSelect*) sel_struct->contents;
 	if (!stsel->from) {
 		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-			     _("SELECT statement has no FROM part"));
+			     "%s", _("SELECT statement has no FROM part"));
 		return FALSE;
 	}
 	if (stsel->from->targets && stsel->from->targets->next) {
 		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-			     _("SELECT statement involves more than one table or expression"));
+			     "%s", _("SELECT statement involves more than one table or expression"));
 		return FALSE;
 	}
 	GdaSqlSelectTarget *target;
 	target = (GdaSqlSelectTarget*) stsel->from->targets->data;
 	if (!target->table_name) {
 		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-			     _("SELECT statement involves more than one table or expression"));
+			     "%s", _("SELECT statement involves more than one table or expression"));
 		return FALSE;
 	}
 	if (!gda_sql_statement_check_validity (sel_struct, cnc, error)) 
@@ -656,15 +760,24 @@ dml_statements_check_select_structure (GdaConnection *cnc, GdaSqlStatement *sel_
 	g_assert (target->validity_meta_object); /* because gda_sql_statement_check_validity() returned TRUE */
 	if (target->validity_meta_object->obj_type != GDA_META_DB_TABLE) {
 		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-			     _("Can only build modification statement for tables"));
+			     "%s", _("Can only build modification statement for tables"));
 		return FALSE;
 	}
 
 	return TRUE;
 }
 
-/*
- * Builds the WHERE condition of the computed DML statement
+/**
+ * gda_compute_unique_table_row_condition
+ * @stsel: a #GdaSqlSelectStatement
+ * @mtable: a #GdaMetaTable
+ * @require_pk: set to TRUE if a primary key ir required
+ * @error: a place to store errors, or %NULL
+ * 
+ * Computes a #GdaSqlExpr expression which can be used in the WHERE clause of an UPDATE
+ * or DELETE statement when a row from the result of the @stsel statement has to be modified.
+ *
+ * Returns: a new #GdaSqlExpr, or %NULL if an error occurred.
  */
 GdaSqlExpr*
 gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTable *mtable, gboolean require_pk, GError **error)
@@ -675,7 +788,7 @@ gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTab
 
 	if (mtable->pk_cols_nb == 0) {
 		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-			     _("Table does not have any primary key"));
+			     "%s", _("Table does not have any primary key"));
 		return NULL;
 	}
 	
@@ -683,7 +796,7 @@ gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTab
 	if (require_pk) {
 		if (mtable->pk_cols_nb == 0) {
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-					     _("Table does not have any primary key"));
+					     "%s", _("Table does not have any primary key"));
 			goto onerror;
 		}
 		else if (mtable->pk_cols_nb > 1) {
@@ -709,7 +822,7 @@ gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTab
 			}
 			if (!sfield) {
 				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-					     _("Table's primary key is not selected"));
+					     "%s", _("Table's primary key is not selected"));
 				goto onerror;
 			}
 			else {
@@ -752,9 +865,20 @@ gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTab
 	return NULL;
 }
 
-/*
- * Statement computation from meta store 
- * @select_stmt: must be a SELECT statement (compound statements not handled)
+/**
+ * gda_compute_dml_statements
+ * @cnc: a #GdaConnection
+ * @select_stmt: a SELECT #GdaStatement (compound statements not handled)
+ * @require_pk: TRUE if the created statement have to use a primary key
+ * @insert_stmt: a place to store the created INSERT statement, or %NULL
+ * @update_stmt: a place to store the created UPDATE statement, or %NULL
+ * @delete_stmt: a place to store the created DELETE statement, or %NULL
+ * @error: a place to store errors, or %NULL
+ *
+ * Creates an INSERT, an UPDATE and a DELETE statement from a SELECT statement
+ * using the database metadata available in @cnc's meta store.
+ * 
+ * returns: TRUE if no error occurred
  */
 gboolean
 gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gboolean require_pk, 
@@ -769,9 +893,12 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 	gboolean retval = TRUE;
 	GdaSqlSelectTarget *target;
 
-	GdaSqlStatementInsert *ist = NULL;
-	GdaSqlStatementUpdate *ust = NULL;
-	GdaSqlStatementDelete *dst = NULL;	
+	GdaSqlStatement *sql_ist = NULL;
+        GdaSqlStatementInsert *ist = NULL;
+        GdaSqlStatement *sql_ust = NULL;
+        GdaSqlStatementUpdate *ust = NULL;
+        GdaSqlStatement *sql_dst = NULL;
+        GdaSqlStatementDelete *dst = NULL;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (GDA_IS_STATEMENT (select_stmt), FALSE);
@@ -800,15 +927,19 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 	
 	/* actual statement structure's computation */        
 	if (insert_stmt) {
-		ist = g_new0 (GdaSqlStatementInsert, 1);
-		GDA_SQL_ANY_PART (ist)->type = GDA_SQL_ANY_STMT_INSERT;
+		sql_ist = gda_sql_statement_new (GDA_SQL_STATEMENT_INSERT);
+		ist = (GdaSqlStatementInsert*) sql_ist->contents;
+		g_assert (GDA_SQL_ANY_PART (ist)->type == GDA_SQL_ANY_STMT_INSERT);
+
 		ist->table = gda_sql_table_new (GDA_SQL_ANY_PART (ist));
 		ist->table->table_name = g_strdup ((gchar *) target->table_name);
 	}
 	
 	if (update_stmt) {
-		ust = g_new0 (GdaSqlStatementUpdate, 1);
-		GDA_SQL_ANY_PART (ust)->type = GDA_SQL_ANY_STMT_UPDATE;
+		sql_ust = gda_sql_statement_new (GDA_SQL_STATEMENT_UPDATE);
+		ust = (GdaSqlStatementUpdate*) sql_ust->contents;
+		g_assert (GDA_SQL_ANY_PART (ust)->type == GDA_SQL_ANY_STMT_UPDATE);
+
 		ust->table = gda_sql_table_new (GDA_SQL_ANY_PART (ust));
 		ust->table->table_name = g_strdup ((gchar *) target->table_name);
 		ust->cond = gda_compute_unique_table_row_condition (stsel, 
@@ -822,8 +953,10 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 	}
         
 	if (delete_stmt) {
-		dst = g_new0 (GdaSqlStatementDelete, 1);
-		GDA_SQL_ANY_PART (dst)->type = GDA_SQL_ANY_STMT_DELETE;
+		sql_dst = gda_sql_statement_new (GDA_SQL_STATEMENT_DELETE);
+		dst = (GdaSqlStatementDelete*) sql_dst->contents;
+		g_assert (GDA_SQL_ANY_PART (dst)->type == GDA_SQL_ANY_STMT_DELETE);
+
 		dst->table = gda_sql_table_new (GDA_SQL_ANY_PART (dst));
 		dst->table->table_name = g_strdup ((gchar *) target->table_name);
 		dst->cond = gda_compute_unique_table_row_condition (stsel, 
@@ -894,58 +1027,31 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 
 	/* finish the statements */
 	if (insert_stmt) {
-		GdaSqlStatement *st;
-
 		if (!ist->fields_list) {
 			/* nothing to insert => don't create statement */
 			/* To translators: this error message occurs when no "INSERT INTO <table> (field1, ...)..." 
 			 * SQL statement can be computed because no table field can be used */
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-				     _("Could not compute any field to insert into"));
+				     "%s", _("Could not compute any field to insert into"));
 			retval = FALSE;
 			goto cleanup;
 		}
 		ist->values_list = g_slist_append (NULL, insert_values_list);
-		st = gda_sql_statement_new (GDA_SQL_STATEMENT_INSERT);
-		st->contents = ist;
-		ist = NULL;
-		ret_insert = g_object_new (GDA_TYPE_STATEMENT, "structure", st, NULL);
-		gda_sql_statement_free (st);
-	}
-	if (update_stmt) {
-		GdaSqlStatement *st;
-		st = gda_sql_statement_new (GDA_SQL_STATEMENT_UPDATE);
-		st->contents = ust;
-		ust = NULL;
-		ret_update = g_object_new (GDA_TYPE_STATEMENT, "structure", st, NULL);
-		gda_sql_statement_free (st);
-	}
-  	if (delete_stmt) {
-		GdaSqlStatement *st;
-		st = gda_sql_statement_new (GDA_SQL_STATEMENT_DELETE);
-		st->contents = dst;
-		dst = NULL;
-		ret_delete = g_object_new (GDA_TYPE_STATEMENT, "structure", st, NULL);
-		gda_sql_statement_free (st);
+		ret_insert = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ist, NULL);
 	}
+	if (update_stmt)
+		ret_update = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ust, NULL);
+  	if (delete_stmt)
+		ret_delete = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_dst, NULL);
 	
  cleanup:
 	gda_sql_statement_free (sel_struct);
-	if (ist) {
-		GdaSqlStatementContentsInfo *cinfo;
-		cinfo = gda_sql_statement_get_contents_infos (GDA_SQL_STATEMENT_INSERT);
-		cinfo->free (ist);
-	}
-	if (ust) {
-		GdaSqlStatementContentsInfo *cinfo;
-		cinfo = gda_sql_statement_get_contents_infos (GDA_SQL_STATEMENT_UPDATE);
-		cinfo->free (ust);
-	}
-	if (dst) {
-		GdaSqlStatementContentsInfo *cinfo;
-		cinfo = gda_sql_statement_get_contents_infos (GDA_SQL_STATEMENT_DELETE);
-		cinfo->free (dst);
-	}
+	if (sql_ist)
+		gda_sql_statement_free (sql_ist);
+	if (sql_ust)
+		gda_sql_statement_free (sql_ust);
+	if (sql_dst)
+		gda_sql_statement_free (sql_dst);
 		
 	if (insert_stmt)
 		*insert_stmt = ret_insert;
@@ -956,6 +1062,59 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 	return retval;
 }
 
+/**
+ * gda_compute_select_statement_from_update
+ * @update_stmt: an UPDATE statement
+ * @error: a place to store errors, or %NULL
+ *
+ * Computes a SELECT statement which selects all the rows the @update_stmt would update. Beware
+ * however that this GdaSqlStatement does not select anything (ie it would be rendered as "SELECT FROM ... WHERE ...")
+ * and before being useable, one needs to add some fields to actually select.
+ *
+ * Returns; a new #GdaStatement if no error occurred, or %NULL otherwise
+ */
+GdaSqlStatement *
+gda_compute_select_statement_from_update (GdaStatement *update_stmt, GError **error)
+{
+	GdaSqlStatement *upd_stmt;
+	GdaSqlStatement *sel_stmt;
+	GdaSqlStatementUpdate *ust;
+	GdaSqlStatementSelect *sst;
+
+	g_return_val_if_fail (update_stmt, NULL);
+	g_object_get (G_OBJECT (update_stmt), "structure", &upd_stmt, NULL);
+	g_return_val_if_fail (upd_stmt, NULL);
+	g_return_val_if_fail (upd_stmt->stmt_type == GDA_SQL_STATEMENT_UPDATE, NULL);
+	
+	ust = (GdaSqlStatementUpdate*) upd_stmt->contents;
+
+	sel_stmt = gda_sql_statement_new (GDA_SQL_STATEMENT_SELECT);
+	sst = (GdaSqlStatementSelect*) sel_stmt->contents;
+	g_assert (GDA_SQL_ANY_PART (sst)->type == GDA_SQL_ANY_STMT_SELECT);
+
+	if (!ust->table || !ust->table->table_name) {
+		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+			     "%s", _("Missing table name in UPDATE statement"));
+		return NULL;
+	}
+
+	/* FROM */
+	GdaSqlSelectTarget *target;
+	sst->from = gda_sql_select_from_new (GDA_SQL_ANY_PART (sst));
+	target = gda_sql_select_target_new (GDA_SQL_ANY_PART (sst->from));
+	sst->from->targets = g_slist_prepend (NULL, target);
+	target->expr = gda_sql_expr_new (GDA_SQL_ANY_PART (target));
+	g_value_set_string ((target->expr->value = gda_value_new (G_TYPE_STRING)), ust->table->table_name);
+
+	/* WHERE */
+	sst->where_cond = gda_sql_expr_copy (ust->cond);
+	GDA_SQL_ANY_PART (sst->where_cond)->parent = GDA_SQL_ANY_PART (sst);
+
+	gda_sql_statement_free (upd_stmt);
+
+	return sel_stmt;
+}
+
 /**
  * gda_identifier_hash
  * @id: an identifier string
@@ -1309,7 +1468,7 @@ copy_ident (const gchar *ident)
 {
 	char *str;
 	gint tlen = strlen (ident);
-	if (_identifier_needs_quotes (ident)) {
+	if (gda_sql_identifier_needs_quotes (ident)) {
 		str = malloc (sizeof (char) * (tlen + 3));
 		*str = '"';
 		strcpy (str+1, ident);
@@ -1333,7 +1492,7 @@ concat_ident (const char *prefix, const gchar *ident)
 	if (prefix)
 		plen = strlen (prefix) + 1;
 
-	if (_identifier_needs_quotes (ident)) {
+	if (gda_sql_identifier_needs_quotes (ident)) {
 		str = malloc (sizeof (char) * (plen + tlen + 3));
 		if (prefix) {
 			strcpy (str, prefix);
@@ -1361,6 +1520,41 @@ concat_ident (const char *prefix, const gchar *ident)
 	return str;
 }
 
+/**
+ * gda_sql_identifier_split
+ * @id: an SQL identifier
+ * 
+ * Splits @id into an array of it sub parts. @id's format has to be "&lt;part&gt;[.&lt;part&gt;[...]]" where
+ * each part is either a text surrounded by double quotes which can contain upper and lower cases or
+ * an SQL identifier in lower case.
+ *
+ * For example the <![CDATA["test.\"ATable\""]]> string will result in the array: <![CDATA[{"test", "\"ATable\"", NULL}]]>
+ *
+ * Returns: a new array of strings, or NULL (use g_strfreev() to free the returned array)
+ */
+gchar **
+gda_sql_identifier_split (const gchar *id)
+{
+	gchar *copy;
+	gchar *remain, *last;
+	GArray *array = NULL;
+
+	g_return_val_if_fail (id && *id, NULL);
+
+	for (copy = g_strdup (id); copy; copy = remain) {
+		if (_split_identifier_string (copy, &remain, &last)) {
+			if (!array)
+				array = g_array_new (TRUE, TRUE, sizeof (gchar *));
+			g_array_prepend_val (array, last);
+		}
+	}
+
+	if (array)
+		return (gchar **) g_array_free (array, FALSE);
+	else
+		return NULL;
+}
+
 /*
  *  RFC 1738 defines that these characters should be escaped, as well
  *  any non-US-ASCII character or anything between 0x00 - 0x1F.
@@ -1540,7 +1734,6 @@ gda_rfc1738_decode (gchar *string)
  * @out_dsn: a place to store the new string containing the &lt;DSN&gt; part
  * @out_username: a place to store the new string containing the &lt;username&gt; part
  * @out_password: a place to store the new string containing the &lt;password&gt; part
- * @error: a place to store errors, or %NULL
  *
  * Extract the DSN, username and password from @string. in @string, the various parts are strings
  * which are expected to be encoded using an RFC 1738 compliant encoding. If they are specified, 
@@ -1590,7 +1783,6 @@ gda_dsn_split (const gchar *string, gchar **out_dsn, gchar **out_username, gchar
  * @out_provider: a place to store the new string containing the &lt;provider&gt; part
  * @out_username: a place to store the new string containing the &lt;username&gt; part
  * @out_password: a place to store the new string containing the &lt;password&gt; part
- * @error: a place to store errors, or %NULL
  *
  * Extract the provider, connection parameters, username and password from @string. 
  * in @string, the various parts are strings
